package it.unimi.dsi.webgraph.examples;


/*		 
 * Copyright (C) 2003 Paolo Boldi and Sebastiano Vigna 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


import it.unimi.dsi.fastutil.ints.IntArrays;
import it.unimi.dsi.fastutil.ints.IntIterator;
import it.unimi.dsi.mg4j.util.ProgressMeter;
import it.unimi.dsi.webgraph.ImmutableGraph;

import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;

/** The main method of this class loads an arbitrary {@link it.unimi.dsi.webgraph.ImmutableGraph}
 * and performs a breadth-first visit of the graph, starting from a given node.
 * It prints the eccentricity of the node (i.e., the maximum distance from the node).
 */

public class BreadthFirst {
	
	private BreadthFirst() {}
	
	static public void main( String arg[] ) throws ClassNotFoundException, IllegalArgumentException, SecurityException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		if ( arg.length != 3 ) {
			System.err.println( "Usage: BreadthFirst GRAPHCLASS BASENAME START" );
			return;
		}
		
		final ProgressMeter pm = new ProgressMeter();
		// We fetch by reflection the class specified by the user
		final Class graphClass = Class.forName( arg[ 0 ], true, ClassLoader.getSystemClassLoader() );
		// We call by reflection the method "load".
		final ImmutableGraph graph = (ImmutableGraph)graphClass.getMethod( "load", new Class[] { CharSequence.class, ProgressMeter.class } ).invoke( graphClass, new Object[] { arg[ 1 ], pm } );
		// We parse the starting node.
		final int startNode = Integer.parseInt( arg[ 2 ] );
		final LinkedList queue = new LinkedList();
		final int[] dist = new int[ graph.numNodes() ];
		IntArrays.fill( dist, Integer.MAX_VALUE ); // Initially, all distances are infinity.
		
		queue.add( new Integer( startNode ) );
		dist[ startNode ] = 0;
			
		int curr = startNode, succ;
		IntIterator successors;
			
		while( ! queue.isEmpty() ) {
			curr = ((Integer)queue.removeFirst()).intValue();
			successors = graph.successors( curr );
			while( successors.hasNext() ) {
				succ = successors.nextInt();
				if ( dist[ succ ] == Integer.MAX_VALUE ) {
					dist[ succ ] = dist[ curr ] + 1;
					queue.add( new Integer( succ ) );
				}
			}
		}
		
		System.out.println( "The eccentricity of node " + startNode + " is " + dist[ curr ] );
	}
	
	
	
}
