package it.unimi.dsi.webgraph.examples;


/*		 
 * Copyright (C) 2003 Paolo Boldi and Sebastiano Vigna 
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */


import it.unimi.dsi.mg4j.util.ProgressMeter;
import it.unimi.dsi.webgraph.ImmutableGraph;
import it.unimi.dsi.webgraph.NodeIterator;

import java.lang.reflect.InvocationTargetException;

/** The main method of this class loads an arbitrary {@link it.unimi.dsi.webgraph.ImmutableGraph}
 * and performs a sequential scan to establish the minimum, maximum and average outdegree.
 */

public class OutdegreeStats {
	
	private OutdegreeStats() {}
	
	static public void main( String arg[] ) throws ClassNotFoundException, IllegalArgumentException, SecurityException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		if ( arg.length != 2 ) {
			System.err.println( "Usage: OutdegreeStats GRAPHCLASS BASENAME" );
			return;
		}
		
		final ProgressMeter pm = new ProgressMeter();
		// We fetch by reflection the class specified by the user
		final Class graphClass = Class.forName( arg[ 0 ], true, ClassLoader.getSystemClassLoader() );
		// We call by reflection the method "load".
		final ImmutableGraph graph = (ImmutableGraph)graphClass.getMethod( "loadSequential", new Class[] { CharSequence.class, ProgressMeter.class } ).invoke( graphClass, new Object[] { arg[ 1 ], pm } );
		
		NodeIterator nodeIterator = graph.nodeIterator();
		int curr, d, maxd = 0, maxNode = 0, mind = Integer.MAX_VALUE, minNode = 0;
		long totd = 0;
			
		while( nodeIterator.hasNext() ) {
			curr = nodeIterator.nextInt();
			d = nodeIterator.outdegree();
				
			if ( d < mind ) {
				mind = d;
				minNode = curr;
			}
			
			if ( d > maxd ){
				maxd = d;
				maxNode = curr; 
			}
			
			totd += d;
		}
		
		System.out.println( "The minimum outdegree is " + mind + ", attained by node " + minNode );
		System.out.println( "The maximum outdegree is " + maxd + ", attained by node " + maxNode );
		System.out.println( "The average outdegree is " + (double)totd / graph.numNodes() );
	}
	
	
	
}
